//*************************************************************************************
// Program:		pfDoubleRate demodulation
// 
//
// Revision history:
//
// V1.02, 2012-01-24:
//    - Procedures pfDoubleRate_GetVersion and pfDoubleRate_DeModulateImageUnmatchedBufferWidth added.
//
//
// SVN:			$Id: pfDoubleRate.h 1674 2013-04-08 11:59:16Z hod $
//*************************************************************************************

#ifndef PFDOUBLERATE_H
#define PFDOUBLERATE_H


#if defined(NODLL) || defined(__linux__)
	#ifdef __cplusplus
		#define PFDOUBLERATE_API  extern "C"
	#else
		#define PFDOUBLERATE_API
	#endif
#else
	#ifdef PFDOUBLERATE_EXPORTS
		#ifdef __cplusplus
			extern "C"
			#define PFDOUBLERATE_API  extern "C" __declspec(dllexport)
		#else
			#define PFDOUBLERATE_API __declspec(dllexport)
		#endif
	#else
		#ifdef __cplusplus
			#define PFDOUBLERATE_API extern "C" __declspec(dllimport)
		#else
			#define PFDOUBLERATE_API __declspec(dllimport)
		#endif
	#endif
#endif


///
/// \brief Gives the version of the pfDoubleRate software.
/// 
///
/// \return Return pfDoubleRate version. Format for version major.minor is: 100*major+minor: 230 = 2.30
///
PFDOUBLERATE_API int pfDoubleRate_GetVersion();


///
/// \brief Set the number of Threads for the demodulation process.
/// 0: No Thread are used, 1-4: Use 1, 2, 3 or 4 Threads.
/// Default is 4. For small ROI, set to 0.
/// 
/// \param[in] nrOfThreads Number of Threads.
///
/// \return Return 0 if OK, else -1
///
PFDOUBLERATE_API int pfDoubleRate_SetNrOfThreads(int nrOfThreads);


///
/// \brief Gives the width of the demodulated image buffer.
/// Call this function before demodulating the image to alloc memory.
/// 
/// \param[in] modImage Pointer to the modulated buffer.
/// \param[out] demodWidth Width of the demodulated buffer.
///
/// \return Return 0 if OK, else -1
///
PFDOUBLERATE_API int pfDoubleRate_GetDeModulatedWidth(unsigned char *modImage, int *demodWidth);


///
/// \brief Gives the width of the modulated image buffer.
/// Call this function before modulate the original image to alloc memory.
/// 
/// \param[in] Width Width of the original image.
/// \param[out] modWidth Width of the modulated buffer.
///
/// \return Return 0 if OK, else -1
///
PFDOUBLERATE_API int pfDoubleRate_GetModulatedWidth(int Width, int *modWidth);


///
/// \brief Demodulate the image. The input image buffer will be demodulated.
/// 
/// \param[out] Image Pointer to the demodulated buffer.
/// \param[in] modImage Pointer to the modulated buffer.
/// \param[in] Width Width of the demodulated image.
/// \param[in] Height Height of both buffers.
/// \param[in] modWidth Width of the modulated buffer.
///
/// \return Return 0 if OK, else -1
///
PFDOUBLERATE_API int pfDoubleRate_DeModulateImage(unsigned char *Image, unsigned char *modImage, int Width, int Height, int modWidth);


///
/// \brief Demodulate the image. The input image buffer will be demodulated.
/// Call this procedure if width of a buffer is not equal to the corresponding image width.
/// 
/// \param[out] Image Pointer to the demodulated buffer.
/// \param[in] modImage Pointer to the modulated buffer.
/// \param[in] Width Width of the demodulated image.
/// \param[in] Height Height of both buffers.
/// \param[in] modWidth Width of the modulated buffer.
/// \param[in] bufWidth Width of the *Image buffer.
/// \param[in] modBufWidth Width of the modulated buffer *modImage.
///
/// \return Return 0 if OK, else -1
///
PFDOUBLERATE_API int pfDoubleRate_DeModulateImageUnmatchedBufferWidth(unsigned char *Image, unsigned char *modImage, int Width, int Height, int modWidth, int bufWidth, int modBufWidth);


// Color functions:


///
/// \brief Gives the width of the demodulated image buffer for color images.
/// Call this function before demudulater the image to alloc memory.
/// 
/// \param[in] modImage Pointer to the modulated buffer (color image).
/// \param[out] demodWidth Width of the demodulated buffer.
///
/// \return Return 0 if OK, else -1
///
PFDOUBLERATE_API int pfDoubleRate_GetDeModulatedWidthColor(unsigned char *modImage, int *demodWidth);


///
/// \brief Gives the width of the modulated image buffer (for color images).
/// Call this function before modulate the original image to alloc memory.
/// 
/// \param[in] Width Width of the original image.
/// \param[out] modWidth Width of the modulated buffer.
///
/// \return Return 0 if OK, else -1
///
PFDOUBLERATE_API int pfDoubleRate_GetModulatedWidthColor(int Width, int *modWidth);


///
/// \brief Demodulate the color image. The input image buffer will be demodulated.
/// 
/// \param[out] Image Pointer to the demodulated buffer (colour image).
/// \param[in] modImage Pointer to the modulated buffer.
/// \param[in] Width Width of the demodulated image.
/// \param[in] Height Height of both buffers.
/// \param[in] modWidth Width of the modulated buffer.
///
/// \return Return 0 if OK, else -1
///
PFDOUBLERATE_API int pfDoubleRate_DeModulateImageColor(unsigned char *Image, unsigned char *modImage, int Width, int Height, int modWidth);


///
/// \brief Demodulate the color image. The input image buffer will be demodulated.
/// Call this procedure if width of a buffer is not equal to the corresponding image width.
/// 
/// \param[out] Image Pointer to the demodulated buffer.
/// \param[in] modImage Pointer to the modulated buffer.
/// \param[in] Width Width of the demodulated image.
/// \param[in] Height Height of both buffers.
/// \param[in] modWidth Width of the modulated buffer.
/// \param[in] bufWidth Width of the *Image buffer.
/// \param[in] modBufWidth Width of the modulated buffer *modImage.
///
/// \return Return 0 if OK, else -1
///
PFDOUBLERATE_API int pfDoubleRate_DeModulateImageUnmatchedBufferWidthColor(unsigned char *Image, unsigned char *modImage, int Width, int Height, int modWidth, int bufWidth, int modBufWidth);


#endif //PFDOUBLERATE_H
