/*
 * pf3dcam error classes for pf3dcamera
 * Copyright (C) 2011 AQSENSE, S.L.
 */
#if !defined (PF3DCAM_ERROR_HPP)
#define PF3DCAM_ERROR_HPP
#include <sal3dpp/Common.hpp>
#ifdef WIN32
#include <pfcam.h>
#endif
#include <pf3dcamC/camera.h>

namespace pf3dcam
{

class Timeout: public std::runtime_error
{
    public:
        Timeout (): std::runtime_error("timeout")
        {}
};

struct PFLibError;

struct Error
{
    Error();
    Error(const sal3d::Error &ex);
    Error(const PFLibError &ex);

    virtual ~Error() throw() {};
    int fill_error(pf3dcam_error *ex) const;
    void throw_if_needed();
    static int set_no_error(struct pf3dcam_error *e);
    static int set_unknown_error(struct pf3dcam_error *e);

    pf3dcam_error e;
};

struct PFLibError : public std::exception
{
    PFLibError(const Error &ex)
        :error(ex.e.pf)
    {
        unsigned int i;
        for(i=0; i < sizeof(text)-1 && ex.e.sal3d.text[i] != '\0'; ++i)
            text[i] = ex.e.sal3d.text[i];
        text[i] = '\0';
    }

    PFLibError(int i, const char *format, ...)
        :error(i)
    {
        va_list ap;

        va_start(ap, format);
        char tmp[sizeof text];
#ifdef WIN32
        vsnprintf_s(tmp, sizeof(tmp), _TRUNCATE, format, ap);
        _snprintf_s
            (text, sizeof text, _TRUNCATE, "%s: %s", tmp,
                pfGetErrorString(error));
#else
        std::vsnprintf(tmp, sizeof(tmp), format, ap);
        std::snprintf
            (text, sizeof text, "%s: %i", tmp,
                error);
#endif
        va_end(ap);
    }

    virtual ~PFLibError() throw() {};

    virtual const char * what() const throw()
    {
        return text;
    }

    int error;
    char text[100];
};

inline Error::Error()
{
    e.is_error = 0;
}

inline Error::Error(const sal3d::Error &ex)
{
    e.is_error = 1;
    e.sal3d = ex.e;
    e.pf = 0;
}

inline Error::Error(const PFLibError &ex)
{
    e.is_error = 1;
    e.pf = ex.error;

    unsigned int i;
    for(i=0; i < sizeof(e.sal3d.text)-1 && ex.text[i] != '\0'; ++i)
        e.sal3d.text[i] = ex.text[i];
    e.sal3d.text[i] = '\0';
}

inline int
Error::fill_error(pf3dcam_error *ex) const
{
    if (ex)
    {
        *ex = e;
    }
    if (e.is_error)
        return -1;
    else
        return 0;
}

inline void
Error::throw_if_needed()
{
    if (e.is_error)
    {
        if (e.pf != 0)
            throw PFLibError(*this);
        if (e.sal3d.value == SAL3D_ERROR_TIMEOUT)
            throw pf3dcam::Timeout();
        else if (e.sal3d.value != SAL3D_ERROR_OK)
            throw sal3d::Error(e.sal3d);
    }
}

inline int
Error::set_no_error(struct pf3dcam_error *e)
{
    if (e)
        e->is_error = 0;
    return SAL3D_ERROR_OK;
}

inline int
Error::set_unknown_error(struct pf3dcam_error *e)
{
    if (e)
    {
        e->is_error = 1;
        return sal3d::Error::set_unknown_error(&e->sal3d);
    }
    else
        return sal3d::Error::set_unknown_error(0);
}

}


#endif
